/*****************************

	i.ching
	
	Based on PatternOracle
	pattern processing engine
	
	version 0.5.0
	
	s y n c h r o n i z m
	synchronizm@mac.com
	
	(c) 2005
	All Rights Reserved.
	
	-------------------
	hexagram.js

	-------------------
	
******************************/

function Kua(y1, y2, y3, y4, y5, y6) {
	
	// functions
	this.initializeKuaState = initializeKuaState;
	this.computeState = computeKuaState;
	this.refreshDataRegisters = refreshDataRegisters;
	this.loadYao = loadYao;
	this.loadHexagramMap =loadHexagramMap;
	this.setState = trace; // overwritten
	
	this.state = new Object();
	this.state_array = new Array();
	this.hexagram_array = new Array();
	
	this.loadHexagramMap();
	
	// create trigrams
	this.lower_trigram = new Trigram(y1, y2, y3);
	this.upper_trigram = new Trigram(y4, y5, y6);
	
	// set handlers
	this.lower_trigram.setState = updateLowerState;
	this.upper_trigram.setState = updateUpperState;
	
	this.initializeKuaState();
	
}

// update state
function computeKuaState() {
	
	// look up
	var lower = this.lower_trigram.state.number;
	var upper = this.upper_trigram.state.number;
	
	// debug
	trace("[Hexagram] lower: " + lower + " upper: " + upper);
	
	if((lower - 0) == lower && (upper - 0) == upper) {
	
		// set state
		this.state = this.hexagram_array[(lower-1)][(upper-1)];
		
		// report
		trace("[Hexagram] (" + this.state.number + ") " + this.state.name);
		
		// update and notify
		this.refreshDataRegisters();
		this.setState(this.state);
	
	}
	
}

// respond to state changes
function updateLowerState(st_obj) {
	
	trace("[Hexagram] " + st_obj.number);
	
	// add state
	//this.state_array[0] = st_obj;
	//this.computeState();
	stable_kua.state_array[0] = st_obj;
	stable_kua.computeState();
	
}

function updateUpperState(st_obj) {
	
	trace("[Hexagram] " + st_obj.number);
	
	// add state
	//this.state_array[1] = st_obj;
	//this.computeState();	
	stable_kua.state_array[1] = st_obj;
	stable_kua.computeState();
}

// Refresh the data registers
function refreshDataRegisters() {
	
	trace("[kua] Rebuilding data registers....");
	
	this.trigrams = new Array();
			
	for(var i=0; i<2; i++) {
		
		// add trigram block
		this.trigrams[i] = new Array();
		
		var trigram = i==0 ? this.lower_trigram : this.upper_trigram;
		
		for(var t=0; t<3; t++) {
			
			// build yao registers 
			this.trigrams[i][t] = new Array();
			
			if(t==0) {
				
				var yao = trigram.bottom_yao;
				
			} else if(t==1) {
				
				var yao = trigram.middle_yao;
				
			} else if(t==2) {
				
				var yao = trigram.top_yao;
				
			}
			
			this.trigrams[i][t] = yao;
			
			/*
			for(var p=0; p<3; p++) {
				
				// build energy registers 
				this.trigrams[i][t][p] = yao;
				
			}
			*/
			
		}
		
	}
	
	trace("[kua] Data registers are now ready");
	
}

// load yao 
function loadYao(yao_num, e1, e2, e3) {
	
	switch (yao_num) {
		
		case 1:
		
			this.lower_trigram.bottom_yao.setEnergies(e1, e2, e3);
			break;
		
		case 2:
		
			this.lower_trigram.middle_yao.setEnergies(e1, e2, e3);
			break;
		
		case 3:
		
			this.lower_trigram.top_yao.setEnergies(e1, e2, e3);
			break;
		
		case 4:
		
			this.upper_trigram.bottom_yao.setEnergies(e1, e2, e3);
			break;
		
		case 5:
		
			this.upper_trigram.middle_yao.setEnergies(e1, e2, e3);
			break;
		
		case 6:
		
			this.upper_trigram.top_yao.setEnergies(e1, e2, e3);
			break;
		
		
	}
	
}

// set hexagram map
function loadHexagramMap() {
	
	this.hexagram_array[0] = new Array();
	this.hexagram_array[0][0] = new Object();
	this.hexagram_array[0][0].name = "Ch'ien - Heaven";
	this.hexagram_array[0][0].number = 1;
	this.hexagram_array[0][1] = new Object();
	this.hexagram_array[0][1].name = "Chueh - Decision";
	this.hexagram_array[0][1].number = 43;
	this.hexagram_array[0][2] = new Object();
	this.hexagram_array[0][2].name = "Ta Yu - Great Possesion";
	this.hexagram_array[0][2].number = 14;
	this.hexagram_array[0][3] = new Object();
	this.hexagram_array[0][3].name = "Ta Chuang - Great Strength";
	this.hexagram_array[0][3].number = 34;
	this.hexagram_array[0][4] = new Object();
	this.hexagram_array[0][4].name = "Hsiao Chu - Minor Domestication";
	this.hexagram_array[0][4].number = 9;
	this.hexagram_array[0][5] = new Object();
	this.hexagram_array[0][5].name = "Hsu - Necessity";
	this.hexagram_array[0][5].number = 5;
	this.hexagram_array[0][6] = new Object();
	this.hexagram_array[0][6].name = "Ta Chu - Great Domestication";
	this.hexagram_array[0][6].number = 26;
	this.hexagram_array[0][7] = new Object();
	this.hexagram_array[0][7].name = "T'ai - Peace";
	this.hexagram_array[0][7].number = 11;
	
	this.hexagram_array[1] = new Array();
	this.hexagram_array[1][0] = new Object();
	this.hexagram_array[1][0].name = "Li - Walking";
	this.hexagram_array[1][0].number = 10;
	this.hexagram_array[1][1] = new Object();
	this.hexagram_array[1][1].name = "Tui - Rejoicing";
	this.hexagram_array[1][1].number = 58;
	this.hexagram_array[1][2] = new Object();
	this.hexagram_array[1][2].name = "K'ui - Opposition";
	this.hexagram_array[1][2].number = 38;
	this.hexagram_array[1][3] = new Object();
	this.hexagram_array[1][3].name = "Kue Mei - The Marriage of the Younger Sister";
	this.hexagram_array[1][3].number = 54;
	this.hexagram_array[1][4] = new Object();
	this.hexagram_array[1][4].name = "Chung Fu - Inner Confidence";
	this.hexagram_array[1][4].number = 61;
	this.hexagram_array[1][5] = new Object();
	this.hexagram_array[1][5].name = "Chieh - Time";
	this.hexagram_array[1][5].number = 60;
	this.hexagram_array[1][6] = new Object();
	this.hexagram_array[1][6].name = "Hsun - Decrease";
	this.hexagram_array[1][6].number = 41;
	this.hexagram_array[1][7] = new Object();
	this.hexagram_array[1][7].name = "Lin - Affability";
	this.hexagram_array[1][7].number = 19;
	
	this.hexagram_array[2] = new Array();
	this.hexagram_array[2][0] = new Object();
	this.hexagram_array[2][0].name = "Tung Jen - The Union of People";
	this.hexagram_array[2][0].number = 13;
	this.hexagram_array[2][1] = new Object();
	this.hexagram_array[2][1].name = "Ko - Change";
	this.hexagram_array[2][1].number = 49;
	this.hexagram_array[2][2] = new Object();
	this.hexagram_array[2][2].name = "Li - Independence";
	this.hexagram_array[2][2].number = 30;
	this.hexagram_array[2][3] = new Object();
	this.hexagram_array[2][3].name = "Feng - Prosperity";
	this.hexagram_array[2][3].number = 55;
	this.hexagram_array[2][4] = new Object();
	this.hexagram_array[2][4].name = "Chia Jen - The Family of Humankind";
	this.hexagram_array[2][4].number = 37;
	this.hexagram_array[2][5] = new Object();
	this.hexagram_array[2][5].name = "Chi Chi - The Attainment of Releif";
	this.hexagram_array[2][5].number = 63;
	this.hexagram_array[2][6] = new Object();
	this.hexagram_array[2][6].name = "Fen - Ornamentation";
	this.hexagram_array[2][6].number = 22;
	this.hexagram_array[2][7] = new Object();
	this.hexagram_array[2][7].name = "Ming I - Death of Light";
	this.hexagram_array[2][7].number = 36;
	
	this.hexagram_array[3] = new Array();
	this.hexagram_array[3][0] = new Object();
	this.hexagram_array[3][0].name = "Wu Wang - Truth";
	this.hexagram_array[3][0].number = 25;
	this.hexagram_array[3][1] = new Object();
	this.hexagram_array[3][1].name = "Sui - Following";
	this.hexagram_array[3][1].number = 17;
	this.hexagram_array[3][2] = new Object();
	this.hexagram_array[3][2].name = "Shih Ho - Biting Through";
	this.hexagram_array[3][2].number = 21;
	this.hexagram_array[3][3] = new Object();
	this.hexagram_array[3][3].name = "Chien - Shaking Force";
	this.hexagram_array[3][3].number = 51;
	this.hexagram_array[3][4] = new Object();
	this.hexagram_array[3][4].name = "I - Increase";
	this.hexagram_array[3][4].number = 42;
	this.hexagram_array[3][5] = new Object();
	this.hexagram_array[3][5].name = "T'un - Initial Struggle";
	this.hexagram_array[3][5].number = 3;
	this.hexagram_array[3][6] = new Object();
	this.hexagram_array[3][6].name = "I - Nourishment"; 
	this.hexagram_array[3][6].number = 27;
	this.hexagram_array[3][7] = new Object();
	this.hexagram_array[3][7].name = "Fu - Going Back Again";
	this.hexagram_array[3][7].number = 24;
	
	this.hexagram_array[4] = new Array();
	this.hexagram_array[4][0] = new Object();
	this.hexagram_array[4][0].name = "Kou - Sexual Union";
	this.hexagram_array[4][0].number = 44;
	this.hexagram_array[4][1] = new Object();
	this.hexagram_array[4][1].name = "Ta Kuo - Great Excess";
	this.hexagram_array[4][1].number = 28;
	this.hexagram_array[4][2] = new Object();
	this.hexagram_array[4][2].name = "Ting - The Tripod";
	this.hexagram_array[4][2].number = 50;
	this.hexagram_array[4][3] = new Object();
	this.hexagram_array[4][3].name = "Heng - Constancy";
	this.hexagram_array[4][3].number = 32;
	this.hexagram_array[4][4] = new Object();
	this.hexagram_array[4][4].name = "Hsuan - Yeilding";
	this.hexagram_array[4][4].number = 57;
	this.hexagram_array[4][5] = new Object();
	this.hexagram_array[4][5].name = "Ching - A Well";
	this.hexagram_array[4][5].number = 48;
	this.hexagram_array[4][6] = new Object();
	this.hexagram_array[4][6].name = "Ku - Poison";
	this.hexagram_array[4][6].number = 18;
	this.hexagram_array[4][7] = new Object();
	this.hexagram_array[4][7].name = "Sheng - Ascent";
	this.hexagram_array[4][7].number = 46;
	
	this.hexagram_array[5] = new Array();
	this.hexagram_array[5][0] = new Object();
	this.hexagram_array[5][0].name = "Sung - Justice";
	this.hexagram_array[5][0].number = 6;
	this.hexagram_array[5][1] = new Object();
	this.hexagram_array[5][1].name = "K'un - Exhaustion";
	this.hexagram_array[5][1].number = 47;
	this.hexagram_array[5][2] = new Object();
	this.hexagram_array[5][2].name = "Wei Chi - No Releif";
	this.hexagram_array[5][2].number = 64;
	this.hexagram_array[5][3] = new Object();
	this.hexagram_array[5][3].name = "Chieh - Loosening Knots";
	this.hexagram_array[5][3].number = 40;
	this.hexagram_array[5][4] = new Object();
	this.hexagram_array[5][4].name = "Huan - Vanishing";
	this.hexagram_array[5][4].number = 59;
	this.hexagram_array[5][5] = new Object();
	this.hexagram_array[5][5].name = "K'an - The Pit";
	this.hexagram_array[5][5].number = 29;
	this.hexagram_array[5][6] = new Object();
	this.hexagram_array[5][6].name = "Meng - Ignorance";
	this.hexagram_array[5][6].number = 4;
	this.hexagram_array[5][7] = new Object();
	this.hexagram_array[5][7].name = "Shih - The People"; 
	this.hexagram_array[5][7].number = 7;
	
	this.hexagram_array[6] = new Array();
	this.hexagram_array[6][0] = new Object();
	this.hexagram_array[6][0].name = "Tun - Retreat";
	this.hexagram_array[6][0].number = 33;
	this.hexagram_array[6][1] = new Object();
	this.hexagram_array[6][1].name = "Hsien - All";
	this.hexagram_array[6][1].number = 31;
	this.hexagram_array[6][2] = new Object();
	this.hexagram_array[6][2].name = "Lu - Travel";
	this.hexagram_array[6][2].number = 56;
	this.hexagram_array[6][3] = new Object();
	this.hexagram_array[6][3].name = "Hsiao Kuo - Minor Excess";
	this.hexagram_array[6][3].number = 62;
	this.hexagram_array[6][4] = new Object();
	this.hexagram_array[6][4].name = "Chien - Advancing by Degrees";
	this.hexagram_array[6][4].number = 53;
	this.hexagram_array[6][5] = new Object();
	this.hexagram_array[6][5].name = "Ch'ien - Misfortune";
	this.hexagram_array[6][5].number = 39;
	this.hexagram_array[6][6] = new Object();
	this.hexagram_array[6][6].name = "Ken - Resistance";
	this.hexagram_array[6][6].number = 52;
	this.hexagram_array[6][7] = new Object();
	this.hexagram_array[6][7].name = "Ch'ien - Modesty";
	this.hexagram_array[6][7].number = 15;
	
	this.hexagram_array[7] = new Array();
	this.hexagram_array[7][0] = new Object();
	this.hexagram_array[7][0].name = "Fao - Negation";
	this.hexagram_array[7][0].number = 12;
	this.hexagram_array[7][1] = new Object();
	this.hexagram_array[7][1].name = "Ts'ui - Coming Together";
	this.hexagram_array[7][1].number = 45;
	this.hexagram_array[7][2] = new Object();
	this.hexagram_array[7][2].name = "Chin - Growth";
	this.hexagram_array[7][2].number = 35;
	this.hexagram_array[7][3] = new Object();
	this.hexagram_array[7][3].name = "Yu - Cheerfulness";
	this.hexagram_array[7][3].number = 16;
	this.hexagram_array[7][4] = new Object();
	this.hexagram_array[7][4].name = "Kuan - Meditation";
	this.hexagram_array[7][4].number = 20;
	this.hexagram_array[7][5] = new Object();
	this.hexagram_array[7][5].name = "Pi - Union";
	this.hexagram_array[7][5].number = 8;
	this.hexagram_array[7][6] = new Object();
	this.hexagram_array[7][6].name = "Pao - Flaying";
	this.hexagram_array[7][6].number = 23;
	this.hexagram_array[7][7] = new Object();
	this.hexagram_array[7][7].name = "K'un - Earth";
	this.hexagram_array[7][7].number = 2;
	
}

// initialize state array
function initializeKuaState() {
	
	this.state_array[0] = this.lower_trigram.state;
	this.state_array[1] = this.upper_trigram.state;
	
}

